import 'dart:io';
import 'dart:convert' show utf8, jsonDecode;

import 'package:resource/resource.dart';

import 'package:keychain/src/keychain_exception.dart';


class Keychain {

  static final _cache = <String, String>{};
  static final environment = Platform.environment;

  static String _checkMap(Map map, String wantKey) {
    for (final k in map.keys) {
      if (k == wantKey && map[k] == String) {
        return map[k];
      } else if (map[k] is Map) {
        final x = _checkMap(map[k], wantKey);
        if (x.isNotEmpty) {
          return x;
        }
      }
    }
    return '';
  }
  

  static String fromEnv(String envVar, {bool allowMissing: false}) {
    if (_cache.containsKey(envVar)) {
      return _cache[envVar];
    } else {
      final r = Platform.environment[envVar];
      if (r == null && !allowMissing) {
        throw KeychainExeception('The environment variable "$envVar" was not found.');
      } else if (r == null) {
        _cache[envVar] = '';
        return _cache[envVar];
      } else {
        _cache[envVar] = r;
        return _cache[envVar];
      }
    }
  }


  static String fromEnvJson(String key, String envVar, {bool allowMissing: false}) {
    final cacheKey = envVar + '|' + key;
    if (_cache.containsKey(cacheKey)) {
      return _cache[cacheKey];
    }
    
    final obj = Platform.environment[envVar];
    
    if (obj == null && !allowMissing) {
      throw KeychainExeception('The environment variable "$envVar" was not found.');
    } else if (obj == null && allowMissing) {
      _cache[cacheKey] = obj;
      return _cache[cacheKey];
    }

    final map = jsonDecode(obj);
    
    final result = _checkMap(map, key);

    if (result.isNotEmpty) {
      _cache[cacheKey] = result;
      return _cache[cacheKey];
    } else if (allowMissing) {
      _cache[cacheKey] = '';
      return _cache[cacheKey];
    } else {
      throw KeychainExeception('The environment variable "$envVar" was not found.');
    }
  }


  static String fromJsonFile(String key, String filePath, {bool allowMissing: false}) {
    var cacheKey = filePath + '|' + key;
    if (_cache.containsKey(cacheKey)) {
      return _cache[cacheKey];
    }

    if (filePath.startsWith(Platform.pathSeparator)) {
      filePath = filePath.substring(1);
    }
    var targetPath = filePath.split(Platform.pathSeparator);
    var scriptPath = Platform.script.pathSegments;
    var index = scriptPath.indexOf(targetPath[0]);
    if (index == -1) {
      throw KeychainExeception('Json file target must relative to a directory on the script path.');
    }
    var pathCopy = List.of(scriptPath);
    pathCopy.replaceRange(index, pathCopy.length, targetPath);
    var u = Uri.file(Platform.pathSeparator + pathCopy.join(Platform.pathSeparator), windows: Platform.isWindows);
    var str = File.fromUri(u).readAsStringSync();

    final map = jsonDecode(str);
    
    final result = _checkMap(map, key);

    if (result.isNotEmpty) {
      _cache[cacheKey] = result;
      return _cache[cacheKey];
    } else if (allowMissing) {
      _cache[cacheKey] = '';
      return _cache[cacheKey];
    } else {
      throw KeychainExeception('The environment variable "$key" from "$filePath" was not found.');
    }

    


    
  }

  static Future<String> fromJsonResource(String key, String resource, {bool allowMissing: false}) async {
    var cacheKey = resource + '|' + key;
    if (_cache.containsKey(cacheKey)) {
      return _cache[cacheKey];
    }
    
    final r = Resource(resource);
    var str = await r.readAsString();

    final map = jsonDecode(str);
    
    final result = _checkMap(map, key);

    if (result.isNotEmpty) {
      _cache[cacheKey] = result;
      return _cache[cacheKey];
    } else if (allowMissing) {
      _cache[cacheKey] = '';
      return _cache[cacheKey];
    } else {
      throw KeychainExeception('The environment variable "$key" in "$resource" was not found.');
    }
    
  }

}